package com.ypsx.yppos.ui.activity

import android.os.Bundle
import androidx.activity.viewModels
import com.afollestad.materialdialogs.MaterialDialog
import com.afollestad.materialdialogs.list.SingleChoiceListener
import com.afollestad.materialdialogs.list.listItemsSingleChoice
import com.elvishew.xlog.XLog
import com.ypsx.base.ext.util.toJson
import com.ypsx.base.network.manager.NetState
import com.ypsx.base.util.AppUtils
import com.ypsx.base.util.DeviceUtils
import com.ypsx.base.util.LogUtils
import com.ypsx.yppos.PosApp
import com.ypsx.yppos.R
import com.ypsx.yppos.databinding.ActivityRegisterBinding
import com.ypsx.yppos.http.data.entity.PosStatus
import com.ypsx.yppos.http.data.request.ApplyPosMachineRequest
import com.ypsx.yppos.room.database.PosDatabase
import com.ypsx.yppos.ui.popup.MessagePopupWindow
import com.ypsx.yppos.utils.CacheUtil
import com.ypsx.base.util.dXLog
import com.ypsx.common.app.base.BaseActivity
import com.ypsx.common.app.ext.getIntent
import com.ypsx.common.app.ext.showServerHostExt
import com.ypsx.common.app.ext.startKtxActivityFinish
import com.ypsx.yppos.viewmodel.request.RequestRegisterViewModel
import com.ypsx.yppos.viewmodel.state.RegisterViewModel

/**
 * 注册页
 */
class RegisterActivity : BaseActivity<RegisterViewModel, ActivityRegisterBinding>() {
    private var exitTime: Long = 0
    private var clickNum = 1
    private val requestMeViewModel: RequestRegisterViewModel by viewModels()

    override fun layoutId(): Int = R.layout.activity_register
    private val hostMap = mapOf(
        "sit开发环境Host" to "https://sit-cpos.ypshengxian.com/",
        "预发Host" to "https://pre-cpos.ypshengxian.com/",
        "线上正式Host" to "https://cpos.ypshengxian.com/",
    )
    private val hosts = hostMap.values.toTypedArray()
    private val names = hostMap.keys.toList()

    override fun initView(savedInstanceState: Bundle?) {
        mDatabind.vm = mViewModel
        mDatabind.click = ProxyClick()

        val state = intent.getBooleanExtra("state", false)
        val posId:String? = intent.getStringExtra("posId")

        if (state) {
            mViewModel.submitState.set(false)
            mViewModel.submitText.set("审核中")
            requestMeViewModel.getById(posId!!)
        }
    }


    inner class ProxyClick {
        fun submit() {
            mViewModel.submitState.set(false)
            mViewModel.submitText.set("审核中")
            val brand = mViewModel.brand
            val model = mViewModel.model
            val ipAddress = mViewModel.ipAddress
            var port = mDatabind.etPort.text.toString()
            if (port.isEmpty()) {
                port = mDatabind.etPort.hint.toString()
            }
            val version = AppUtils.getVersionCode().toString()
            val macAddress = DeviceUtils.getMacAddress()
            val applyPosMachineRequest = ApplyPosMachineRequest(
                brand,
                ipAddress,
                macAddress,
                port.toInt(),
                version,
                model,
                "8G", "/",
            )
            XLog.d("提交申请信息：${applyPosMachineRequest.toJson()}")


//            showLoading("请求网络中...")
            requestMeViewModel.applyPos(applyPosMachineRequest)
        }

        fun combosClick(){
            showServerHostExt(CacheUtil.getHost()) {
                CacheUtil.setHost(it)
                CacheUtil.clearAll()
                PosDatabase.clearAllTables()
                PosApp.instance.restartApplication()
            }
        }
    }



    override fun createObserver() {
        requestMeViewModel.posDetails.observe(this, { data ->
            val id = data.id
            val status = data.status
            requestMeViewModel.savePosConfig(data)
            when (status) {
                PosStatus.APPLY.name -> {
                    LogUtils.debugInfo(status)
                    requestMeViewModel.getById(id)
                }
                PosStatus.ACTIVE.name -> {
                    startLogin()
                }
                PosStatus.APPLY_FAILED.name -> {
                    val applyFailedMessage = data.applyFailedMessage
                    LogUtils.debugInfo("错误原因：${applyFailedMessage}")
                    showError("提示", "您的审核未通过，请重新提交", "我知道了")
                }
                PosStatus.DISABLED.name -> {
                    val messagePopupWindow = MessagePopupWindow(this)
                    messagePopupWindow.showPopupWindow()
                    messagePopupWindow.setData("提示", "该POS机已停用，是否将删除本地数据", "取消", "确定", false)
                    messagePopupWindow.setMessageClick {
                        messagePopupWindow.dismiss()
                        CacheUtil.setUpdateTimeBigger("")
                        CacheUtil.setLoginResponse(null)
                    }
                }
            }


        })

    }

    private fun startLogin() {
        startKtxActivityFinish<LoginActivity>()

    }

    private fun showError(title: String, content: String, sure: String) {
        val messagePopupWindow = MessagePopupWindow(this)
        messagePopupWindow.showPopupWindow()
        messagePopupWindow.setData(title, content, sure = sure, gone = false)

    }

    override fun onNetworkStateChanged(netState: NetState) {
        super.onNetworkStateChanged(netState)
        if (netState.isSuccess) {
            "我特么终于有网了啊!".dXLog()
        } else {
            "我特么怎么断网了!".dXLog()
        }
    }


    private fun showServerHostPopWindow() {
        //选中的host
        val checkHost = CacheUtil.getHost()
        var checkedIndex = hosts.indexOf(checkHost)
        if (checkedIndex < 0) checkedIndex = names.size - 1//也就是研发Native那个
        //启用输入的host
        MaterialDialog(this).title(text = "切换Server接口")
            .cancelable(false)
            .cancelOnTouchOutside(false)
            .listItemsSingleChoice(
                items = names,
                initialSelection = checkedIndex,
                selection = object : SingleChoiceListener {
                    override fun invoke(dialog: MaterialDialog, index: Int, text: CharSequence) {
                        //如果选择了研发native，index就会超过host的index
                        if (index < hosts.size) {
                            CacheUtil.setHost(hosts[index])
                            CacheUtil.clearAll()
                            PosDatabase.clearAllTables()
                            PosApp.instance.restartApplication()
                        }
                    }
                })
            .positiveButton(text = "确定")
            .negativeButton(text = "取消")
            .show()
    }

}